@@ -27,6 +27,10 @@
 --
 --
 -- $Log: read_md5_file.c,v $
+-- Revision 1.5  2008/08/31 18:25:44  jsedwards
+-- Change for new parameters to nwos_initialize_objectify without key stuff
+-- and eliminate calle to nwos_set_root_object because initialize does that now.
+--
 -- Revision 1.4  2008/05/08 11:58:12  jsedwards
 -- Fixed so that it will accept an asterisk before the file name.
 --
@@ -86,7 +90,6 @@ static uint8 convert_char_to_hex_value(char c)
 
 int main(int argc, char* argv[])
 {
-    ObjRef root_ref;
     ObjRef ref;
     int i;
     int entry = 0;
@@ -187,20 +190,8 @@ int main(int argc, char* argv[])
 
     nwos_log_arguments(argc, argv);
 
-    nwos_initialize_objectify(NULL, 0, 0, PUBLIC, NULL);
+    nwos_initialize_objectify(PUBLIC, NULL);
 
-    printf("next_reference: %02x%02x%02x%02x\n", 
-	   nwos_next_public_ref.id[0],
-	   nwos_next_public_ref.id[1],
-	   nwos_next_public_ref.id[2],
-	   nwos_next_public_ref.id[3]);
-
-    root_ref.id[0] = 0;
-    root_ref.id[1] = 0;
-    root_ref.id[2] = 0;
-    root_ref.id[3] = 1;
-
-    nwos_set_root_object(&root_ref);
 
     for (i = 0; i < entry; i++)
     {
