@@ -1,6 +1,6 @@
 /*
 --             This file is part of the New World OS project
---                Copyright (C) 2006, 2007  QRW Software
+--                 Copyright (C) 2006-2008  QRW Software
 --           J. Scott Edwards - j.scott.edwards.nwos@gmail.com 
 --                      http://www.qrwsoftware.com
 --                      http://nwos.sourceforge.com
@@ -27,6 +27,28 @@
 --
 --
 -- $Log: export_c_structs.c,v $
+-- Revision 1.15  2008/09/01 20:34:29  jsedwards
+-- Change for new nwos_initialize_objectify calling convention and remove call
+-- to nwos_set_root_object, nwos_initialize_objectify calls it now.
+--
+-- Revision 1.14  2008/08/31 21:53:51  jsedwards
+-- Added an assert around calls to nwos_read_variable_sized_object_from_disk
+-- and nwos_read_object_from_disk because now they return false when they fail
+-- instead of asserting themselves.
+--
+-- Revision 1.13  2008/08/30 12:05:03  jsedwards
+-- Changed for new parameters to nwos_initialize_objectify.
+--
+-- Revision 1.12  2008/03/12 11:41:53  jsedwards
+-- Added call to nwos_log_arguments(0, NULL) to disable logging.
+--
+-- Revision 1.11  2008/02/16 12:15:30  jsedwards
+-- Change AccessType to PUBLIC.
+--
+-- Revision 1.10  2007/08/17 15:54:32  jsedwards
+-- Change check for "Sha1" to "Sha" and any number, so it will match "Sha1",
+-- "Sha256", or "Sha512".
+--
 -- Revision 1.9  2007/07/27 17:09:55  jsedwards
 -- Changed to deal with class definitions that have revisions.
 --
@@ -93,7 +115,7 @@ char *class_to_type(ObjRef* class_ref)
     C_struct_Class_Definition* class_def_obj_ptr = (C_struct_Class_Definition*)kludge;
     char name[32];
 
-    nwos_read_variable_sized_object_from_disk(class_ref, kludge, sizeof(kludge), &get_class_object_size);
+    assert(nwos_read_variable_sized_object_from_disk(class_ref, kludge, sizeof(kludge), &get_class_object_size));
     nwos_name_to_string(&class_def_obj_ptr->name, name, sizeof(name));
 
     if (strcmp(name, "Object Reference") == 0) return "ObjRef";
@@ -117,7 +139,7 @@ void output_feature(ObjRef* feature_ref)
     char label[32];
     int i;
 
-    nwos_read_object_from_disk(feature_ref, &feature_obj, sizeof(feature_obj));
+    assert(nwos_read_object_from_disk(feature_ref, &feature_obj, sizeof(feature_obj)));
 
     nwos_name_to_string(&feature_obj.label, label, sizeof(label));
 
@@ -151,7 +173,7 @@ void output_class(ObjRef* class_ref)
     int i;
     int revision;
 
-    nwos_read_variable_sized_object_from_disk(class_ref, kludge, sizeof(kludge), &get_class_object_size);
+    assert(nwos_read_variable_sized_object_from_disk(class_ref, kludge, sizeof(kludge), &get_class_object_size));
     nwos_name_to_string(&class_def_obj_ptr->name, name, sizeof(name));
 
     /* don't create structures for the built in types */
@@ -176,7 +198,7 @@ void output_class(ObjRef* class_ref)
 	{
 	    name[1] = 'D';
 	}
-	else if (name[0] == 'S' && name[1] == 'h' && name[2] == 'a' && name[3] == '1')
+	else if (name[0] == 'S' && name[1] == 'h' && name[2] == 'a' && isdigit(name[3]))
 	{
 	    name[1] = 'H';
 	    name[2] = 'A';
@@ -202,7 +224,7 @@ void output_class(ObjRef* class_ref)
 	    {
 		revision++;
 
-		nwos_read_variable_sized_object_from_disk(&class_def_obj_ptr->header.object.prev_version, kludge, sizeof(kludge), &get_class_object_size);
+		assert(nwos_read_variable_sized_object_from_disk(&class_def_obj_ptr->header.object.prev_version, kludge, sizeof(kludge), &get_class_object_size));
 	    }
 
 	    printf("} C_struct_%s_%03d;\n\n", name, revision);
@@ -214,28 +236,22 @@ int main(int argc, char* argv[])
 {
     uint8 kludge[FILE_BLOCK_SIZE];
     C_struct_Class_Definition* class_def_obj_ptr = (C_struct_Class_Definition*)kludge;
-    ObjRef root_ref;
     ObjRef obj_class;
     ReferenceList* ref_list;
     int num_refs;
     int i;
 
+    nwos_log_arguments(0, NULL);   /* disable logging */
+
     if (argc == 1)
     {
-	nwos_initialize_objectify(NULL, 0, 0, 0, NULL);    /* public only */ 
+	nwos_initialize_objectify(PUBLIC, NULL);    /* public only */ 
     }
     else
     {
 	fprintf(stderr, "usage: %s [public.objects.file]\n", argv[0]);
     }
 
-    root_ref.id[0] = 0;
-    root_ref.id[1] = 0;
-    root_ref.id[2] = 0;
-    root_ref.id[3] = 1;
-
-    nwos_set_root_object(&root_ref);
-
 
     printf("/* This file was created by the 'export_c_structs' program from the objectify */\n");
     printf("/* public objects.  It should not be edited to make additions or changes, the */\n");
@@ -243,7 +259,7 @@ int main(int argc, char* argv[])
     printf("\n\n");
 
 
-    nwos_read_variable_sized_object_from_disk(&nwos_public_class_definition_class_ref, kludge, sizeof(kludge), &get_class_object_size);
+    assert(nwos_read_variable_sized_object_from_disk(&nwos_public_class_definition_class_ref, kludge, sizeof(kludge), &get_class_object_size));
 
     ref_list = nwos_malloc_reference_list(&class_def_obj_ptr->header.object.references);
 
