@@ -27,6 +27,27 @@
 --
 --
 -- $Log: list_discs.c,v $
+-- Revision 1.24  2008/09/01 03:13:42  jsedwards
+-- Change for new nwos_initialize_objectify calling convention (doesn't pass
+-- back root_object_reference anymore) and removed call to nwos_set_root_object
+-- because initialize calls it now.
+--
+-- Revision 1.23  2008/08/31 21:53:52  jsedwards
+-- Added an assert around calls to nwos_read_variable_sized_object_from_disk
+-- and nwos_read_object_from_disk because now they return false when they fail
+-- instead of asserting themselves.
+--
+-- Revision 1.22  2008/08/30 12:46:13  jsedwards
+-- Removed code and variables to read pass phrase and pass it to initialize,
+-- and change parameters passed to initialize.
+--
+-- Revision 1.21  2008/05/18 17:10:09  jsedwards
+-- Feature Request #1966442 - added ability to pass disc IDs with wildcards on
+-- command line.
+--
+-- Revision 1.20  2008/04/09 12:35:05  jsedwards
+-- Added call to nwos_log_arguments to disable logging.
+--
 -- Revision 1.19  2008/02/03 01:05:23  jsedwards
 -- Changed DEFAULT_TYPE_RO to READ_ONLY.
 --
@@ -96,6 +117,7 @@
 */
 
 
+#include <fnmatch.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -113,11 +135,6 @@ static size_t get_path_object_size(void* file_path_obj)
 
 int main(int argc, char* argv[])
 {
-    ObjRef root_object_ref;
-    uint8 big_key[16 + 8 + 4];
-    uint8 bf_key[16];
-    uint32 linear;
-    uint32 serial;
     C_struct_Class_Definition class_def_obj;
     C_struct_Disc_Copy copy_obj;
     C_struct_Storage_Location location_obj;
@@ -142,39 +159,53 @@ int main(int argc, char* argv[])
     uint32 file_size;
     int list_class_num_refs;
     int disc_list_num_refs;
+    int argn = 1;
     int i;
     int j;
     int count;
     int copies;
     bool do_contents = false;
+    bool match;
+    bool8* found_args = NULL;
 
-    if (argc > 1)
+
+    nwos_log_arguments(0, NULL);   /* disable logging */
+
+    for (i = 1; i < argc; i++)
     {
-	if (argc == 2 && strcmp(argv[1], "--contents") == 0)
+	if (strcmp(argv[i], "--contents") == 0)
 	{
+	    if (i != 1)
+	    {
+		fprintf(stderr, "--content option must preceed any disc ids\n");
+		exit(1);
+	    }
+
 	    do_contents = true;
+	    argn = 2;
 	}
-	else
+	else if (*argv[i] == '-')
 	{
-	    fprintf(stderr, "usage: %s [--contents]\n", argv[0]);
+	    fprintf(stderr, "unknown option: %s\n", argv[i]);
+	    fprintf(stderr, "usage: %s [--contents] [disc id(s)]\n", argv[0]);
 	    exit(1);
 	}
+	else if (found_args == NULL)    /* need to allocate array for flags */
+	{
+	    found_args = malloc(argc);
+
+	    if (found_args == NULL)
+	    {
+		perror("allocating found_args array");
+		exit(1);
+	    }
+
+	    memset(found_args, 0, argc);
+	}
     }
 
-    printf("\n");
+    nwos_initialize_objectify(READ_ONLY, DEFAULT_FILE);
 
-    nwos_get_key_from_password(big_key, sizeof(big_key));
-
-    printf("\n");
-
-    memcpy(bf_key, big_key, 16);
-    linear = ((uint32)big_key[16] << 24) | ((uint32)big_key[17] << 16) | ((uint32)big_key[18] << 8) | (uint32)big_key[19];
-    memcpy(root_object_ref.id, big_key+20, 4);
-    serial = ((uint32)big_key[24] << 24) | ((uint32)big_key[25] << 16) | ((uint32)big_key[26] << 8) | (uint32)big_key[27];
-
-    nwos_initialize_objectify(bf_key, linear, serial, READ_ONLY, DEFAULT_FILE);
-
-    nwos_set_root_object(&root_object_ref);
 
     if (!nwos_find_private_class_definition("FILE PATH", &path_class_ref))
     {
@@ -220,102 +251,146 @@ int main(int argc, char* argv[])
 
 	if (is_same_object(&object_class, &list_class_ref))
 	{
-	    nwos_read_variable_sized_object_from_disk(&list_class_ref_list->references[i], kludge, sizeof(kludge), &nwos_get_disc_list_object_size);
+	    assert(nwos_read_variable_sized_object_from_disk(&list_class_ref_list->references[i], kludge, sizeof(kludge), &nwos_get_disc_list_object_size));
 
 	    memcpy(id_str, ptr_list_obj->id, 12);
 	    id_str[12] = '\0';
 
-	    count = nwos_decode_variable_sized_count(ptr_list_obj->count);
+	    match = true;   /* list all files if no ids were given on command line */
 
-	    printf("disc: %s  files: %u\n", id_str, count);
-
-	    disc_list_ref_list = nwos_malloc_reference_list(&ptr_list_obj->header.object.references);
-
-	    disc_list_num_refs = disc_list_ref_list->common_header.num_refs;
-
-	    copies = 0;
-
-	    for (j = 0; j < disc_list_num_refs; j++)
+	    if (argn < argc)    /* disc ids were passed to the program on the command line */
 	    {
-		nwos_get_object_class(&disc_list_ref_list->references[j], &object_class);
+		match = false;
 
-		if (is_same_object(&object_class, &copy_class_ref))
+		for (j = argn; j < argc; j++)
 		{
-		    copies++;
-
-		    nwos_read_object_from_disk(&disc_list_ref_list->references[j], &copy_obj, sizeof(copy_obj));
-
-		    nwos_get_object_class(&copy_obj.location, &object_class);
-
-		    if (is_void_reference(&copy_obj.header.object.next_version))
+		    if (fnmatch(argv[j], id_str, 0) == 0)   /* found a match */
 		    {
-			if (is_same_object(&object_class, &loc_class_ref))
-			{
-			    nwos_read_object_from_disk(&copy_obj.location, &location_obj, sizeof(location_obj));
-			    copy_reference(&name_ref, &location_obj.name);
-			}
-			else if (is_same_object(&object_class, &person_class_ref))
-			{
-			    nwos_read_object_from_disk(&copy_obj.location, &person_obj, sizeof(person_obj));
-			    if (is_void_reference(&person_obj.goes_by))
-			    {
-				copy_reference(&name_ref, &person_obj.first_name);
-			    }
-			    else
-			    {
-				copy_reference(&name_ref, &person_obj.goes_by);
-			    }
-			}
-			else
-			{
-			    fprintf(stderr, "Error: unknown location class: %02x%02x%02x%02x\n",
-				    copy_obj.location.id[0],
-				    copy_obj.location.id[1],
-				    copy_obj.location.id[2],
-				    copy_obj.location.id[3]);
-			}
-
-			assert(nwos_name_to_string(&name_ref, name_str, sizeof(name_str)));
-
-			printf(" copy number: %d  location: %s\n", copy_obj.copy_number, name_str);
+			match = true;
+			found_args[j] = true;
+			break;
 		    }
 		}
 	    }
 
-	    nwos_free_reference_list(disc_list_ref_list);
-	    disc_list_ref_list = NULL;
-
-	    if (copies == 0)
+	    if (match)   /* found a match */
 	    {
-		printf("  NO copies logged!\n");
-	    }
+		count = nwos_decode_variable_sized_count(ptr_list_obj->count);
 
-	    if (do_contents)    /* list all the files as well */
-	    {
-		for (j = 0; j < count; j++)
+		printf("disc: %s  files: %u\n", id_str, count);
+
+		disc_list_ref_list = nwos_malloc_reference_list(&ptr_list_obj->header.object.references);
+
+		disc_list_num_refs = disc_list_ref_list->common_header.num_refs;
+
+		copies = 0;
+
+		for (j = 0; j < disc_list_num_refs; j++)
 		{
-		    nwos_read_object_from_disk(&ptr_list_obj->files[j], &assoc_obj, sizeof(assoc_obj));
+		    nwos_get_object_class(&disc_list_ref_list->references[j], &object_class);
 
-		    nwos_read_variable_sized_object_from_disk(&assoc_obj.path, cludge, sizeof(cludge), &get_path_object_size);
+		    if (is_same_object(&object_class, &copy_class_ref))
+		    {
+			copies++;
 
-		    memcpy(name_str, ptr_path_obj->storage, ptr_path_obj->count);
-		    name_str[ptr_path_obj->count] = '\0';
+			assert(nwos_read_object_from_disk(&disc_list_ref_list->references[j], &copy_obj, sizeof(copy_obj)));
 
-		    nwos_read_object_from_disk(&assoc_obj.file, &file_obj, sizeof(file_obj));
+			nwos_get_object_class(&copy_obj.location, &object_class);
 
-		    file_size = (file_obj.size[0] << 24) | (file_obj.size[1] << 16) | (file_obj.size[2] << 8) | (file_obj.size[3]);
+			if (is_void_reference(&copy_obj.header.object.next_version))
+			{
+			    if (is_same_object(&object_class, &loc_class_ref))
+			    {
+				assert(nwos_read_object_from_disk(&copy_obj.location, &location_obj, sizeof(location_obj)));
+				copy_reference(&name_ref, &location_obj.name);
+			    }
+			    else if (is_same_object(&object_class, &person_class_ref))
+			    {
+				assert(nwos_read_object_from_disk(&copy_obj.location, &person_obj, sizeof(person_obj)));
+				if (is_void_reference(&person_obj.goes_by))
+				{
+				    copy_reference(&name_ref, &person_obj.first_name);
+				}
+				else
+				{
+				    copy_reference(&name_ref, &person_obj.goes_by);
+				}
+			    }
+			    else
+			    {
+				fprintf(stderr, "Error: unknown location class: %02x%02x%02x%02x\n",
+					copy_obj.location.id[0],
+					copy_obj.location.id[1],
+					copy_obj.location.id[2],
+					copy_obj.location.id[3]);
+			    }
 
-		    printf("   %s %u\n", name_str, file_size);
+			    assert(nwos_name_to_string(&name_ref, name_str, sizeof(name_str)));
+
+			    printf(" copy number: %d  location: %s\n", copy_obj.copy_number, name_str);
+			}
+		    }
+		}
+
+		nwos_free_reference_list(disc_list_ref_list);
+		disc_list_ref_list = NULL;
+
+		if (copies == 0)
+		{
+		    printf("  NO copies logged!\n");
+		}
+
+		if (do_contents)    /* list all the files as well */
+		{
+		    for (j = 0; j < count; j++)
+		    {
+			assert(nwos_read_object_from_disk(&ptr_list_obj->files[j], &assoc_obj, sizeof(assoc_obj)));
+
+			assert(nwos_read_variable_sized_object_from_disk(&assoc_obj.path, cludge, sizeof(cludge), &get_path_object_size));
+
+			memcpy(name_str, ptr_path_obj->storage, ptr_path_obj->count);
+			name_str[ptr_path_obj->count] = '\0';
+
+			assert(nwos_read_object_from_disk(&assoc_obj.file, &file_obj, sizeof(file_obj)));
+
+			file_size = (file_obj.size[0] << 24) | (file_obj.size[1] << 16) | (file_obj.size[2] << 8) | (file_obj.size[3]);
+
+			printf("   %s %u\n", name_str, file_size);
+		    }
 		}
 	    }
 	}
     }
 
+    printf("\n");
+
     nwos_free_reference_list(list_class_ref_list);
     list_class_ref_list = NULL;
 
     nwos_terminate_objectify();
 
+    if (found_args != NULL)
+    {
+	match = true;      /* see if they all matched */
+
+	for (i = argn; i < argc; i++)
+	{
+	    if (!found_args[i])
+	    {
+		fprintf(stderr, "WARNING: no IDs found that match: %s\n", argv[i]);
+		match = false;
+	    }
+	}
+
+	if (!match)
+	{
+	    fprintf(stderr, "\n");
+	}
+
+	free(found_args);
+	found_args = NULL;
+    }
+
     return 0;
 }
 
