@@ -27,6 +27,42 @@
 --
 --
 -- $Log: list_files.c,v $
+-- Revision 1.35  2008/09/01 03:13:43  jsedwards
+-- Change for new nwos_initialize_objectify calling convention (doesn't pass
+-- back root_object_reference anymore) and removed call to nwos_set_root_object
+-- because initialize calls it now.
+--
+-- Revision 1.34  2008/08/31 21:53:52  jsedwards
+-- Added an assert around calls to nwos_read_variable_sized_object_from_disk
+-- and nwos_read_object_from_disk because now they return false when they fail
+-- instead of asserting themselves.
+--
+-- Revision 1.33  2008/08/30 12:46:13  jsedwards
+-- Removed code and variables to read pass phrase and pass it to initialize,
+-- and change parameters passed to initialize.
+--
+-- Revision 1.32  2008/06/01 04:10:49  jsedwards
+-- Fix assert to verify the correct reference for the previous version.
+--
+-- Revision 1.31  2008/05/31 14:37:54  jsedwards
+-- Fix so that revision is always initialized.
+--
+-- Revision 1.30  2008/05/31 14:32:07  jsedwards
+-- Fixed so that old name gets the ;0 revision NOT the new name.
+--
+-- Revision 1.29  2008/05/31 13:47:56  jsedwards
+-- Changed code to flag old revision files that were a name change, with a ;0
+-- revision.
+--
+-- Revision 1.28  2008/05/22 11:35:58  jsedwards
+-- Changed so that it deals with pre Alpha_28 revised files, but warns of
+-- problem at the end.
+--
+-- Revision 1.27  2008/05/21 13:26:40  jsedwards
+-- Added code to deal with revisied file links being in PATH AND FILE
+-- ASSOCIATION objects instead of FILE objects.  Still need to figure out
+-- how to deal with reading compressed files that are the old way.
+--
 -- Revision 1.26  2008/04/09 12:35:21  jsedwards
 -- Added call to nwos_log_arguments to disable logging.
 --
@@ -152,8 +188,8 @@ void print_md5(ObjRef* file_ref)
     C_struct_MD5sum md5_object;
     int j;
 
-    nwos_read_object_from_disk(file_ref, &file_obj, sizeof(file_obj));  /* read the file object */
-    nwos_read_object_from_disk(&file_obj.md5sum, &md5_object, sizeof(md5_object));
+    assert(nwos_read_object_from_disk(file_ref, &file_obj, sizeof(file_obj)));  /* read the file object */
+    assert(nwos_read_object_from_disk(&file_obj.md5sum, &md5_object, sizeof(md5_object)));
 
     for (j = 0; j < MD5_DIGEST_SIZE; j++) printf("%02x", md5_object.md5sum[j]);
     printf("  ");
@@ -166,7 +202,7 @@ void print_sha1(ObjRef* file_ref)
     C_struct_SHA1sum sha1_object;
     int j;
 
-    nwos_read_object_from_disk(file_ref, &file_obj, sizeof(file_obj));  /* read the file object */
+    assert(nwos_read_object_from_disk(file_ref, &file_obj, sizeof(file_obj)));  /* read the file object */
 
     if (is_void_reference(&file_obj.sha1sum))
     {
@@ -177,7 +213,7 @@ void print_sha1(ObjRef* file_ref)
     }
     else
     {
-	nwos_read_object_from_disk(&file_obj.sha1sum, &sha1_object, sizeof(sha1_object));
+	assert(nwos_read_object_from_disk(&file_obj.sha1sum, &sha1_object, sizeof(sha1_object)));
 
 	for (j = 0; j < SHA1_DIGEST_SIZE; j++) printf("%02x", sha1_object.sha1sum[j]);
 	printf("  ");
@@ -191,7 +227,7 @@ void print_sha256(ObjRef* file_ref)
     C_struct_SHA256sum sha256_object;
     int j;
 
-    nwos_read_object_from_disk(file_ref, &file_obj, sizeof(file_obj));  /* read the file object */
+    assert(nwos_read_object_from_disk(file_ref, &file_obj, sizeof(file_obj)));  /* read the file object */
 
     if (is_void_reference(&file_obj.sha256sum))
     {
@@ -202,7 +238,7 @@ void print_sha256(ObjRef* file_ref)
     }
     else
     {
-	nwos_read_object_from_disk(&file_obj.sha256sum, &sha256_object, sizeof(sha256_object));
+	assert(nwos_read_object_from_disk(&file_obj.sha256sum, &sha256_object, sizeof(sha256_object)));
 
 	for (j = 0; j < SHA256_DIGEST_SIZE; j++) printf("%02x", sha256_object.sha256sum[j]);
     }
@@ -217,7 +253,7 @@ void print_sha512(ObjRef* file_ref)
     C_struct_SHA512sum sha512_object;
     int j;
 
-    nwos_read_object_from_disk(file_ref, &file_obj, sizeof(file_obj));  /* read the file object */
+    assert(nwos_read_object_from_disk(file_ref, &file_obj, sizeof(file_obj)));  /* read the file object */
 
     if (is_void_reference(&file_obj.sha512sum))
     {
@@ -225,7 +261,7 @@ void print_sha512(ObjRef* file_ref)
     }
     else
     {
-	nwos_read_object_from_disk(&file_obj.sha512sum, &sha512_object, sizeof(sha512_object));
+	assert(nwos_read_object_from_disk(&file_obj.sha512sum, &sha512_object, sizeof(sha512_object)));
 
 	for (j = 0; j < SHA512_DIGEST_SIZE; j++) printf("%02x", sha512_object.sha512sum[j]);
     }
@@ -255,13 +291,10 @@ void print_time(TimeStamp ts)
 
 int main(int argc, char* argv[])
 {
-    ObjRef root_object_ref;
-    uint8 big_key[16 + 8 + 4];
-    uint8 bf_key[16];
-    uint32 linear;
-    uint32 serial;
     C_struct_Class_Definition class_def_obj;
     C_struct_Path_And_File_Association assoc_obj;
+    C_struct_Path_And_File_Association prev_assoc_obj;
+    C_struct_Path_And_File_Association next_assoc_obj;
     uint8 kludge[FILE_BLOCK_SIZE];
     C_struct_File_Path* ptr_path_obj = (C_struct_File_Path*)kludge;
     C_struct_File file_obj;
@@ -276,6 +309,7 @@ int main(int argc, char* argv[])
     int i;
     int j;
     int revision;
+    int old_file_revisions = 0;
     char name[256];
     bool do_md5 = false;
     bool do_sha1 = false;
@@ -376,14 +410,7 @@ int main(int argc, char* argv[])
 
     if (do_public)
     {
-	nwos_initialize_objectify(NULL, 0, 0, PUBLIC, NULL);
-
-	root_object_ref.id[0] = 0;
-	root_object_ref.id[1] = 0;
-	root_object_ref.id[2] = 0;
-	root_object_ref.id[3] = 1;
-
-	nwos_set_root_object(&root_object_ref);
+	nwos_initialize_objectify(PUBLIC, NULL);
 
 	if (!nwos_find_public_class_definition("PATH AND FILE ASSOCIATION", &assoc_class_ref))
 	{
@@ -393,16 +420,7 @@ int main(int argc, char* argv[])
     }
     else
     {
-	nwos_get_key_from_password(big_key, sizeof(big_key));
-
-	memcpy(bf_key, big_key, 16);
-	linear = ((uint32)big_key[16] << 24) | ((uint32)big_key[17] << 16) | ((uint32)big_key[18] << 8) | (uint32)big_key[19];
-	memcpy(root_object_ref.id, big_key+20, 4);
-	serial = ((uint32)big_key[24] << 24) | ((uint32)big_key[25] << 16) | ((uint32)big_key[26] << 8) | (uint32)big_key[27];
-
-	nwos_initialize_objectify(bf_key, linear, serial, READ_ONLY, path);
-
-	nwos_set_root_object(&root_object_ref);
+	nwos_initialize_objectify(READ_ONLY, path);
 
 	if (!nwos_find_private_class_definition("PATH AND FILE ASSOCIATION", &assoc_class_ref))
 	{
@@ -425,13 +443,13 @@ int main(int argc, char* argv[])
 
 	if (is_same_object(&object_class, &assoc_class_ref))
 	  {
-	    nwos_read_object_from_disk(&ref_list->references[i], &assoc_obj, sizeof(assoc_obj));
+	    assert(nwos_read_object_from_disk(&ref_list->references[i], &assoc_obj, sizeof(assoc_obj)));
 
 	    match = false;
 
-	    if (do_list_old_names || is_void_reference(&assoc_obj.header.object.next_version))
+	    if (do_list_old_names || do_revision || is_void_reference(&assoc_obj.header.object.next_version))
 	    {
-		nwos_read_variable_sized_object_from_disk(&assoc_obj.path, kludge, sizeof(kludge), &get_path_object_size);
+		assert(nwos_read_variable_sized_object_from_disk(&assoc_obj.path, kludge, sizeof(kludge), &get_path_object_size));
 
 		/* remember ptr_path_obj points to the kludge buffer */
 
@@ -482,17 +500,82 @@ int main(int argc, char* argv[])
 
 		if (do_revision)
 		{
-		    nwos_read_object_from_disk(&assoc_obj.file, &file_obj, sizeof(file_obj));
 		    revision = 1;
-		    while (!is_void_reference(&file_obj.header.object.prev_version))
+
+		    /* This is complicated because of my earlier bad descision about handling file revisions */
+		    /* (putting them in the file objects).  Then when I moved them to the path and file      */
+		    /* association objects, that made multiple possiblities for what the revision means.     */
+		    /*                                                                                       */
+		    /* 1) Rename - code in rename_file will not allow a path that had more than one file     */
+		    /*    reference be renamed so we can assume that if a the path changed it is a rename.   */
+		    /*                                                                                       */
+		    /* 2) Revision - a file could have been renamed before, but not after because of the     */
+		    /*    code in rename_file.                                                               */
+
+		    /* Check to see if this is an old name for a file */
+		    if (!is_void_reference(&assoc_obj.header.object.next_version))
 		    {
-			copy_reference(&next_ref, &file_obj.header.common.id);
-			copy_reference(&prev_ref, &file_obj.header.object.prev_version);
-			nwos_read_object_from_disk(&prev_ref, &file_obj, sizeof(file_obj));
-			assert(is_same_object(&file_obj.header.object.next_version, &next_ref));
-			revision++;
+			assert(nwos_read_object_from_disk(&assoc_obj.header.object.next_version, &next_assoc_obj, sizeof(next_assoc_obj)));
+			assert(is_same_object(&next_assoc_obj.header.object.prev_version, &ref_list->references[i]));
+			if (!is_same_object(&next_assoc_obj.path, &assoc_obj.path))  /* path changed, must be a rename */
+			{
+			    assert(is_same_object(&next_assoc_obj.file, &assoc_obj.file));
+			    assert(is_void_reference(&assoc_obj.header.object.prev_version));
+			    revision = 0;    /* flag old names as revision 0 */
+			}
+		    }
+
+		    /* Check to see if it has a new style revision (previous/next links in the association object */
+		    if (!is_void_reference(&assoc_obj.header.object.prev_version))
+		    {
+			copy_reference(&prev_ref, &assoc_obj.header.object.prev_version);
+			copy_reference(&next_ref, &assoc_obj.header.common.id);
+			while (!is_void_reference(&prev_ref))
+			{
+			    assert(nwos_read_object_from_disk(&prev_ref, &prev_assoc_obj, sizeof(prev_assoc_obj)));
+			    assert(is_same_object(&prev_assoc_obj.header.object.next_version, &next_ref));
+
+			    /* don't count it if it was a name change */
+			    if (is_same_object(&prev_assoc_obj.path, &assoc_obj.path))
+			    {
+				revision++;
+			    }
+
+			    copy_reference(&prev_ref, &prev_assoc_obj.header.object.prev_version);
+			    copy_reference(&next_ref, &prev_assoc_obj.header.common.id);
+			}
+		    }
+		    /* Check to see if it has an old style revision (previous/next links in the file object) */
+		    /* NOTE: no need to check for an old style revision if the file was renamed because we   */
+		    /*       know the file is the same (assert verified above) so we will do the revision    */
+		    /*       when we get to the new name.                                                    */
+		    else if (revision != 0)
+		    {
+			assert(nwos_read_object_from_disk(&assoc_obj.file, &file_obj, sizeof(file_obj)));
+
+			while (!is_void_reference(&file_obj.header.object.prev_version))
+			{
+			    copy_reference(&next_ref, &file_obj.header.common.id);
+			    copy_reference(&prev_ref, &file_obj.header.object.prev_version);
+			    assert(nwos_read_object_from_disk(&prev_ref, &file_obj, sizeof(file_obj)));
+			    assert(is_same_object(&file_obj.header.object.next_version, &next_ref));
+			    if (revision > 0)
+			    {
+				revision = -revision;    /* make it negative to indicate it is an old style */
+			    }
+			    revision--;
+			}
+		    }
+
+		    if (revision < 0)   /* old style */
+		    {
+			printf(";%d*", -revision);
+			old_file_revisions++;
+		    }
+		    else
+		    {
+			printf(";%d", revision);
 		    }
-		    printf(";%d", revision);
 		}
 
 		if (do_time)
@@ -510,6 +593,39 @@ int main(int argc, char* argv[])
     }
 
     printf("Number of files: %d\n", num_files);
+    fflush(stdout);
+
+    if (old_file_revisions > 0)
+    {
+	fprintf(stderr, "\n");
+
+	if (path == DEFAULT_FILE)   /* reading from normal archive */
+	{
+	    fprintf(stderr, "WARNING: the %d files with an asterisk (*) following the revision number are\n",
+		    old_file_revisions);
+	    fprintf(stderr, "         using an old method for storing the file revision information (pre\n");
+	    fprintf(stderr, "         Alpha_28).  You should run the program: update_files_0028 on this\n");
+	    fprintf(stderr, "         archive to update them to the current method.\n");
+	}
+	else if (memcmp(nwos_disk_header.version_string, "0028", 4) < 0)    /* reading from old compressed file */
+	{
+	    fprintf(stderr, "NOTE: the %d files with an asterisk (*) following the revision number used\n",
+		    old_file_revisions);
+	    fprintf(stderr, "      an old method for storing the file revision information (pre Alpha_28).\n");
+	    fprintf(stderr, "      Since this compressed file predates Alpha_28 no action is necessary.\n");
+	}
+	else  /* reading from newer compressed file */
+	{
+	    fprintf(stderr, "WARNING: the %d files with an asterisk (*) following the revision number used\n",
+		    old_file_revisions);
+	    fprintf(stderr, "         an old method for storing the file revision information (pre\n");
+	    fprintf(stderr, "         Alpha_28).  Since this compressed file postdates that change, you\n");
+	    fprintf(stderr, "         should verify that the archive it was created from has been updated.\n");
+	    fprintf(stderr, "         You can verify it by running the program 'update_files_0028' on it.\n");
+	}
+
+	fprintf(stderr, "\n");
+    }
 
     nwos_free_reference_list(ref_list);
     ref_list = NULL;
