@@ -27,6 +27,12 @@
 --
 --
 -- $Log: log.c,v $
+-- Revision 1.17  2008/08/24 14:39:36  jsedwards
+-- Added code to disable logging if log file path is NULL or empty.
+--
+-- Revision 1.16  2008/04/26 11:36:13  jsedwards
+-- Added 'const' to char* file_name parameter in the four log_*sum functions.
+--
 -- Revision 1.15  2008/03/14 13:11:55  jsedwards
 -- Added nwos_log_sha512sum function.
 --
@@ -172,6 +178,12 @@ void nwos_log(char* str)
     {
 	path = nwos_get_log_file_path();
 
+	if (path == NULL || *path == '\0')       /* no log file specified */
+	{
+	    log_disabled = true;
+	    return;
+	}
+	
 	/* NOTE: this is a bad thing to do, it temporarily modifies */
 	/* the string returned by nwos_get_log_file_path.           */
         /* We cheat the system by calling strchr.                   */
@@ -266,7 +278,7 @@ void nwos_log_arguments(int argc, char* argv[])
 
 static char hexdigit[16] = { '0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'a', 'b', 'c', 'd', 'e', 'f' };
 
-void nwos_log_md5sum(char* file_name, uint8 md5_digest[MD5_DIGEST_SIZE])
+void nwos_log_md5sum(const char* file_name, uint8 md5_digest[MD5_DIGEST_SIZE])
 {
     char msg[12 + FILENAME_MAX + MD5_DIGEST_SIZE * 2] = "MD5 (";
     int prefix_len;
@@ -291,7 +303,7 @@ void nwos_log_md5sum(char* file_name, uint8 md5_digest[MD5_DIGEST_SIZE])
 }
 
 
-void nwos_log_sha1sum(char* file_name, uint8 sha1_digest[SHA1_DIGEST_SIZE])
+void nwos_log_sha1sum(const char* file_name, uint8 sha1_digest[SHA1_DIGEST_SIZE])
 {
     char msg[12 + FILENAME_MAX + SHA1_DIGEST_SIZE * 2] = "SHA1 (";
     int prefix_len;
@@ -316,7 +328,7 @@ void nwos_log_sha1sum(char* file_name, uint8 sha1_digest[SHA1_DIGEST_SIZE])
 }
 
 
-void nwos_log_sha256sum(char* file_name, uint8 sha256_digest[SHA256_DIGEST_SIZE])
+void nwos_log_sha256sum(const char* file_name, uint8 sha256_digest[SHA256_DIGEST_SIZE])
 {
     char msg[16 + FILENAME_MAX + SHA256_DIGEST_SIZE * 2] = "SHA256 (";
     int prefix_len;
@@ -341,7 +353,7 @@ void nwos_log_sha256sum(char* file_name, uint8 sha256_digest[SHA256_DIGEST_SIZE]
 }
 
 
-void nwos_log_sha512sum(char* file_name, uint8 sha512_digest[SHA512_DIGEST_SIZE])
+void nwos_log_sha512sum(const char* file_name, uint8 sha512_digest[SHA512_DIGEST_SIZE])
 {
     char msg[16 + FILENAME_MAX + SHA512_DIGEST_SIZE * 2] = "SHA512 (";
     int prefix_len;
