@@ -1,23 +1,50 @@
-/*             This file is part of the New World OS project
---                 Copyright (C) 2005-2007  QRW Software
+/*
+--             This file is part of the New World OS project
+--                 Copyright (C) 2005-2008  QRW Software
 --           J. Scott Edwards - j.scott.edwards.nwos@gmail.com 
 --                      http://www.qrwsoftware.com
 --                      http://nwos.sourceforge.com
 --
--- NWOS is free software;  you can redistribute it and/or modify it under the
--- terms of the GNU General Public License  as published by the Free Software
--- Foundation; either version 2, or (at your option) any later version.  This
--- software is distributed with the hope that it will be useful,  but WITHOUT
--- ANY WARRANTY;  without  even the  implied warranty  of MERCHANTABILITY  or
--- FITNESS FOR A PARTICULAR PURPOSE.   See the GNU General Public License for
--- more  details.  You should have received a copy  of the GNU General Public
--- License along with this package;  see the file LICENSE.  If not, write to:
+--   This program is free software: you can redistribute it and/or modify
+--   it under the terms of the GNU General Public License as published by
+--   the Free Software Foundation, either version 3 of the License, or
+--   (at your option) any later version.
+--
+--   This program is distributed in the hope that it will be useful,
+--   but WITHOUT ANY WARRANTY; without even the implied warranty of
+--   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+--   GNU General Public License for more details.
+--
+--   You should have received a copy of the GNU General Public License
+--   along with this program, in the file LICENSE.  If not, see 
+--   <http://www.gnu.org/licenses/>.
+--
+--   You can also contact me via paper mail at:
+--
+--      QRW Software
+--      P.O. Box 27511
+--      Salt Lake City, UT 84127-0511, USA.
 --
---      Free Software Foundation, Inc.
---      59 Temple Place - Suite 330
---      Boston, MA 02111-1307, USA.
 --
 -- $Log: person.c,v $
+-- Revision 1.25  2008/09/01 00:10:54  jsedwards
+-- Fix copyright year.  NO code changes.
+--
+-- Revision 1.24  2008/08/31 20:33:18  jsedwards
+-- Add assert around calls to nwos_read_object_from_disk because it now
+-- returns false when it fails, whereas before it would assert itself.
+--
+-- Revision 1.23  2007/07/29 19:46:00  jsedwards
+-- Fix so variable isn't compiled because code that uses it isn't compiled to
+-- eliminate compiler warning.
+--
+-- Revision 1.22  2007/07/12 03:11:47  jsedwards
+-- Add nwos_add_birthday function to add a birthday to a person that doesn't
+-- have one.
+--
+-- Revision 1.21  2007/07/01 19:44:12  jsedwards
+-- Upgrade to GPLv3.
+--
 -- Revision 1.20  2007/05/05 14:43:45  jsedwards
 -- Changed add_person to verify public names first.
 --
@@ -223,7 +250,7 @@ void nwos_find_public_gender(char* gender, ObjRef* ref)
 	assert(nwos_find_word("male", &gender_class_ref, &english_lang_ref, &word_ref));
     }
 
-    nwos_read_object_from_disk(&word_ref, &word_obj, sizeof(word_obj));
+    assert(nwos_read_object_from_disk(&word_ref, &word_obj, sizeof(word_obj)));
 
     memcpy(ref, &word_obj.thing, sizeof(ObjRef));
 }
@@ -259,7 +286,7 @@ bool nwos_find_private_gender(char* gender, ObjRef* ref)
 
 	    if (is_same_object(&class_ref, &gender_class_ref))
 	    {
-		nwos_read_object_from_disk(&ref_list->references[i], &gender_obj, sizeof(gender_obj));
+		assert(nwos_read_object_from_disk(&ref_list->references[i], &gender_obj, sizeof(gender_obj)));
 
 		if (is_same_object(&gender_obj.header.object.clone_of, &public_gender_ref))
 		{
@@ -297,7 +324,7 @@ void nwos_find_or_create_private_gender(char* gender, ObjRef* ref)
 
 	nwos_find_or_create_private_class_definition("GENDER", &private_gender_class_ref);
 
-	nwos_read_object_from_disk(&public_gender_ref, &gender_obj, sizeof(gender_obj));
+	assert(nwos_read_object_from_disk(&public_gender_ref, &gender_obj, sizeof(gender_obj)));
 
 	nwos_generate_new_completely_random_id(ref);
 
@@ -377,7 +404,7 @@ ObjCreateResult nwos_create_ssn(ObjRef* person, char* number, ObjRef* ref)
 
     for (i = 0; i < num_years; i++)
     {
-	nwos_read_object_from_disk(&ref_list->references[i], kludge, sizeof(kludge));
+	assert(nwos_read_object_from_disk(&ref_list->references[i], kludge, sizeof(kludge)));
 
 	/* remember the ptr_year_obj points to the kludge buffer */
 
@@ -653,6 +680,123 @@ void nwos_add_person()
 }
 
 
+void nwos_add_birthday(char* name, char* date)
+{
+    bool date_ok;
+    ObjRef person_ref;
+    int month, day, year;
+    C_struct_Person person_obj;
+#if 0
+    char temp[16];
+#endif
+
+    date_ok = (sscanf(date, "%d/%d/%d", &month, &day, &year) == 3);
+    if (!date_ok) date_ok = (sscanf(date, "%d-%d-%d", &month, &day, &year) == 3);
+
+    if (!date_ok)
+    {
+	printf("Date must be of the format MM/DD/YYYY or MM-DD-YYYY\n");
+    }
+    else
+    {
+	date_ok = (1582 <= year && year <= 9999);    /* 1582 start of Gregorian calendar */
+
+	if (!date_ok)
+	{
+	    printf("Year must be in the range 1582 to 9999\n");
+	}
+	else
+	{
+	    date_ok = (1 <= month && month <= 12);
+
+	    if (!date_ok)
+	    {
+		printf("Month must be in the range 1 to 12\n");
+	    }
+	    else
+	    {
+		switch (month)
+		{
+		  case 2:
+		    if (nwos_is_leap_year(year))
+		    {
+			date_ok = (1 <= day && day <= 29);
+
+			if (!date_ok)
+			{
+			    printf("Day must be in the range 1 to 29 because %d is a leap year\n", year);
+			}
+		    }
+		    else
+		    {
+			date_ok = (1 <= day && day <= 28);
+
+			if (!date_ok)
+			{
+			    printf("Day must be in the range 1 to 28 because %d is not a leap year\n", year);
+			}
+		    }
+		    break;
+
+		  case 4:
+		  case 6:
+		  case 9:
+		  case 11:
+		    date_ok = (1 <= day && day <= 30);
+		    if (!date_ok)
+		    {
+			printf("Day must be in the range 1 to 30\n");
+		    }
+		    break;
+
+		  default:
+		    date_ok = (1 <= day && day <= 31);
+		    if (!date_ok)
+		    {
+			printf("Day must be in the range 1 to 31\n");
+		    }
+		    break;
+		}
+	    }
+	}
+    }
+
+    if (!date_ok) return;
+
+    if (!nwos_find_person(name, &person_ref))
+    {
+	printf("I'm sorry I can't find %s in the database.\n", name);
+	return;
+    }
+
+    assert(nwos_read_object_from_disk(&person_ref, &person_obj, sizeof(person_obj)));
+
+    if (!is_void_reference(&person_obj.birth_date))
+    {
+	printf("This version cannot change a birthday, SORRY!\n");
+	return;
+#if 0
+	nwos_date_to_string(&person_obj.birth_date, temp, sizeof(temp));
+
+	printf("%s already has a birthday set: %s\n", name, temp);
+
+	if (nwos_ask_yes_or_no(NULL, "overwrite"))
+	{
+	    /* need to make sure we can delete reference from old date */
+	}
+#endif
+    }  
+	
+    nwos_create_date(year, month, day, &person_obj.birth_date);
+
+    nwos_crc32_calculate((uint8*) &person_obj.gender, sizeof(person_obj) - sizeof(EveryObject), person_obj.header.common.data_chksum);
+
+    nwos_overwrite_object_to_disk(&person_ref, &person_obj, sizeof(person_obj));
+
+    nwos_add_to_references(&person_ref, &person_obj.birth_date);
+}
+
+
 bool nwos_any_persons_named(char* name)
 {
     ObjRef name_ref;
@@ -770,7 +914,7 @@ bool nwos_find_person(char* name, ObjRef* person_ref)
 
 	    for (j = 0; j < person_count; j++)    /* see if we already have this one */
 	    {
-		nwos_read_object_from_disk(&person_refs[j], &person_obj, sizeof(person_obj));
+		assert(nwos_read_object_from_disk(&person_refs[j], &person_obj, sizeof(person_obj)));
 
 		nwos_name_to_string(&person_obj.first_name, temp, sizeof(temp));
 		printf("  %d: %s ", j, temp);
@@ -845,7 +989,7 @@ void nwos_list_persons()
 
 	if (is_same_object(&object_class, &person_class_ref))
 	{
-	    nwos_read_object_from_disk(&ref_list->references[i], &person_obj, sizeof(person_obj));
+	    assert(nwos_read_object_from_disk(&ref_list->references[i], &person_obj, sizeof(person_obj)));
 
 	    nwos_name_to_string(&person_obj.last_name, name, sizeof(name));
 
@@ -883,7 +1027,7 @@ void nwos_describe_person(char* name)
     }
     else
     {
-	nwos_read_object_from_disk(&person_ref, &person_obj, sizeof(person_obj));
+	assert(nwos_read_object_from_disk(&person_ref, &person_obj, sizeof(person_obj)));
 
 	printf("full name: ");
 
